<?xml version="1.0" encoding="utf-8"?>
<!--
   Copyright (C) 2023 KylinSoft Co., Ltd.

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 3, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, see &lt;http://www.gnu.org/licenses/&gt;.
 -->
<!DOCTYPE TS>
<TS version="2.1">
<context>
    <name>BiometricAuthWidget</name>
    <message>
        <location filename="../BiometricAuth/biometricauthwidget.cpp" line="100"/>
        <source>Current device: </source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../BiometricAuth/biometricauthwidget.cpp" line="163"/>
        <source>Identify failed, Please retry.</source>
        <translation type="unfinished"></translation>
    </message>
</context>
<context>
    <name>BiometricDevicesWidget</name>
    <message>
        <location filename="../BiometricAuth/biometricdeviceswidget.cpp" line="50"/>
        <source>Please select the biometric device</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../BiometricAuth/biometricdeviceswidget.cpp" line="55"/>
        <source>Device type:</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../BiometricAuth/biometricdeviceswidget.cpp" line="72"/>
        <source>Device name:</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../BiometricAuth/biometricdeviceswidget.cpp" line="82"/>
        <source>OK</source>
        <translation type="unfinished"></translation>
    </message>
</context>
<context>
    <name>DeviceType</name>
    <message>
        <location filename="../BiometricAuth/biometricdeviceinfo.cpp" line="37"/>
        <source>FingerPrint</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../BiometricAuth/biometricdeviceinfo.cpp" line="39"/>
        <source>FingerVein</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../BiometricAuth/biometricdeviceinfo.cpp" line="41"/>
        <source>Iris</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../BiometricAuth/biometricdeviceinfo.cpp" line="43"/>
        <source>Face</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../BiometricAuth/biometricdeviceinfo.cpp" line="45"/>
        <source>VoicePrint</source>
        <translation type="unfinished"></translation>
    </message>
</context>
<context>
    <name>FakeDialog</name>
    <message>
        <location filename="../common/fakedialog.cpp" line="63"/>
        <source>cancel</source>
        <translation type="unfinished"></translation>
    </message>
</context>
<context>
    <name>GreeterWrapper</name>
    <message>
        <location filename="../greeter/greeterwrapper.cpp" line="100"/>
        <source>failed to start session.</source>
        <translation type="unfinished"></translation>
    </message>
</context>
<context>
    <name>KeyboardWidget</name>
    <message>
        <location filename="../VirtualKeyboard/src/keyboardwidget.ui" line="29"/>
        <source>KeyboardWidget</source>
        <translation type="unfinished"></translation>
    </message>
</context>
<context>
    <name>LoginWindow</name>
    <message>
        <location filename="../greeter/loginwindow.cpp" line="267"/>
        <location filename="../greeter/loginwindow.cpp" line="575"/>
        <source>Login</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../greeter/loginwindow.cpp" line="312"/>
        <source>logged in</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../greeter/loginwindow.cpp" line="392"/>
        <source>login</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../greeter/loginwindow.cpp" line="480"/>
        <source>Password: </source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../greeter/loginwindow.cpp" line="482"/>
        <source>Username</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../greeter/loginwindow.cpp" line="539"/>
        <source>Incorrect user name, please input again</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../greeter/loginwindow.cpp" line="544"/>
        <location filename="../greeter/loginwindow.cpp" line="549"/>
        <source>Incorrect password, please input again</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../greeter/loginwindow.cpp" line="580"/>
        <location filename="../greeter/loginwindow.cpp" line="789"/>
        <source>Retry</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../greeter/loginwindow.cpp" line="754"/>
        <source>Biometric Authentication</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../greeter/loginwindow.cpp" line="767"/>
        <source>Password Authentication</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../greeter/loginwindow.cpp" line="779"/>
        <source>Other Devices</source>
        <translation type="unfinished"></translation>
    </message>
</context>
<context>
    <name>MainWindow</name>
    <message>
        <location filename="../display-switch/mainwindow.ui" line="14"/>
        <location filename="../display-switch/ui_mainwindow.h" line="127"/>
        <source>MainWindow</source>
        <translation type="unfinished"></translation>
    </message>
</context>
<context>
    <name>PowerManager</name>
    <message>
        <location filename="../greeter/powerwindow.cpp" line="131"/>
        <source>Switch User</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../greeter/powerwindow.cpp" line="144"/>
        <source>Suspend</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../greeter/powerwindow.cpp" line="158"/>
        <source>Restart</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../greeter/powerwindow.cpp" line="172"/>
        <source>Power Off</source>
        <translation type="unfinished"></translation>
    </message>
</context>
<context>
    <name>UsersModel</name>
    <message>
        <location filename="../greeter/usersmodel.cpp" line="42"/>
        <location filename="../greeter/usersmodel.cpp" line="44"/>
        <source>Guest</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../greeter/usersmodel.cpp" line="52"/>
        <source>Guest Session</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../greeter/usersmodel.cpp" line="70"/>
        <location filename="../greeter/usersmodel.cpp" line="72"/>
        <location filename="../greeter/usersmodel.cpp" line="79"/>
        <source>Login</source>
        <translation type="unfinished"></translation>
    </message>
</context>
</TS>
