/*
 * Copyright (C) 2023 KylinSoftCo., Ltd.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, see <http://www.gnu.org/licenses/>.
 *
**/
#include "modebutton.h"
#include <QStylePainter>
#include <QStyleOptionButton>

ModeButton::ModeButton(QWidget *parent)
    : QPushButton(parent)
{

}

void ModeButton::paintEvent(QPaintEvent *event)
{
    QStylePainter p(this);
    QStyleOptionButton option;
    initStyleOption(&option);
    option.state = QStyle::State_Enabled;
    p.drawControl(QStyle::CE_PushButton, option);
}

