<?xml version="1.0" encoding="utf-8"?>
<!--
   Copyright (C) 2023 KylinSoft Co., Ltd.

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 3, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, see &lt;http://www.gnu.org/licenses/&gt;.
 --><!DOCTYPE TS>
<TS version="2.1" language="vi" sourcelanguage="en">
<context>
    <name>AgreementWindow</name>
    <message>
        <location filename="../greeter/agreementwindow.cpp" line="32"/>
        <source>I know</source>
        <translation>Tôi đã biết</translation>
    </message>
</context>
<context>
    <name>BatteryWidget</name>
    <message>
        <location filename="../greeter/batterywidget.cpp" line="173"/>
        <source>BatteryMode</source>
        <translation>Chế độ pin</translation>
    </message>
    <message>
        <location filename="../greeter/batterywidget.cpp" line="170"/>
        <source>PowerMode</source>
        <translation>Chế độ nguồn điện</translation>
    </message>
    <message>
        <location filename="../greeter/batterywidget.cpp" line="164"/>
        <source>Charging...</source>
        <translation>Sạc...</translation>
    </message>
    <message>
        <location filename="../greeter/batterywidget.cpp" line="166"/>
        <source>fully charged</source>
        <translation>Đã sạc đầy</translation>
    </message>
</context>
<context>
    <name>BiometricAuthWidget</name>
    <message>
        <source>Current device: </source>
        <translation type="vanished">当前设备：</translation>
    </message>
    <message>
        <source>Identify failed, Please retry.</source>
        <translation type="vanished">识别失败，请重试</translation>
    </message>
</context>
<context>
    <name>BiometricDevicesWidget</name>
    <message>
        <source>Please select the biometric device</source>
        <translation type="vanished">请选择生物设备</translation>
    </message>
    <message>
        <source>Device type:</source>
        <translation type="vanished">设备类型：</translation>
    </message>
    <message>
        <source>Device name:</source>
        <translation type="vanished">设备型号：</translation>
    </message>
    <message>
        <source>OK</source>
        <translation type="vanished">确定</translation>
    </message>
</context>
<context>
    <name>CharsMoreWidget</name>
    <message>
        <source>Return</source>
        <translation type="vanished">返回</translation>
    </message>
    <message>
        <location filename="../VirtualKeyboard/src/charsmorewidget.cpp" line="166"/>
        <source>&amp;&amp;?!</source>
        <translation>&amp;&amp;?!</translation>
    </message>
</context>
<context>
    <name>CharsWidget</name>
    <message>
        <location filename="../VirtualKeyboard/src/charswidget.cpp" line="97"/>
        <source>More</source>
        <translation>Nhiều hơn</translation>
    </message>
    <message>
        <location filename="../VirtualKeyboard/src/charswidget.cpp" line="111"/>
        <source>ABC</source>
        <translation>ABC</translation>
    </message>
    <message>
        <location filename="../VirtualKeyboard/src/charswidget.cpp" line="124"/>
        <source>123</source>
        <translation>123</translation>
    </message>
    <message>
        <source>Return</source>
        <translation type="vanished">返回</translation>
    </message>
    <message>
        <source>Number</source>
        <translation type="vanished">数字</translation>
    </message>
</context>
<context>
    <name>ConfForm</name>
    <message>
        <source>edit network</source>
        <translation type="vanished">编辑网络设置</translation>
    </message>
    <message>
        <source>LAN name: </source>
        <translation type="vanished">网络名称：</translation>
    </message>
    <message>
        <source>Method: </source>
        <translation type="vanished">编辑IP设置: </translation>
    </message>
    <message>
        <source>Address: </source>
        <translation type="vanished">IP地址: </translation>
    </message>
    <message>
        <source>Netmask: </source>
        <translation type="vanished">子网掩码: </translation>
    </message>
    <message>
        <source>Gateway: </source>
        <translation type="vanished">默认网关: </translation>
    </message>
    <message>
        <source>DNS 1: </source>
        <translation type="vanished">首选DNS: </translation>
    </message>
    <message>
        <source>DNS 2: </source>
        <translation type="vanished">备选DNS: </translation>
    </message>
    <message>
        <source>Edit Conn</source>
        <translation type="vanished">网络设置</translation>
    </message>
    <message>
        <source>Auto(DHCP)</source>
        <translation type="vanished">自动(DHCP)</translation>
    </message>
    <message>
        <source>Manual</source>
        <translation type="vanished">手动</translation>
    </message>
    <message>
        <source>Cancel</source>
        <translation type="vanished">取消</translation>
    </message>
    <message>
        <source>Save</source>
        <translation type="vanished">保存</translation>
    </message>
    <message>
        <source>Ok</source>
        <translation type="vanished">确定</translation>
    </message>
    <message>
        <source>Can not create new wired network for without wired card</source>
        <translation type="vanished">缺少有线网卡 无法新建网络</translation>
    </message>
    <message>
        <source>New network already created</source>
        <translation type="vanished">已创建新的有线网络</translation>
    </message>
    <message>
        <source>New network settings already finished</source>
        <translation type="vanished">新的网络配置已经完成</translation>
    </message>
    <message>
        <source>Edit Network</source>
        <translation type="vanished">编辑网络设置</translation>
    </message>
    <message>
        <source>Add Wired Network</source>
        <translation type="vanished">新建有线网络</translation>
    </message>
</context>
<context>
    <name>DeviceType</name>
    <message>
        <location filename="../BiometricAuth/biometricdeviceinfo.cpp" line="38"/>
        <source>FingerPrint</source>
        <translation>Dấu vân tay</translation>
    </message>
    <message>
        <location filename="../BiometricAuth/biometricdeviceinfo.cpp" line="40"/>
        <source>FingerVein</source>
        <translation>Tĩnh mạch ngón tay</translation>
    </message>
    <message>
        <location filename="../BiometricAuth/biometricdeviceinfo.cpp" line="42"/>
        <source>Iris</source>
        <translation>Võng mạc</translation>
    </message>
    <message>
        <location filename="../BiometricAuth/biometricdeviceinfo.cpp" line="44"/>
        <source>Face</source>
        <translation>Mặt</translation>
    </message>
    <message>
        <location filename="../BiometricAuth/biometricdeviceinfo.cpp" line="46"/>
        <source>VoicePrint</source>
        <translation>Bản in giọng nói</translation>
    </message>
    <message>
        <location filename="../BiometricAuth/biometricdeviceinfo.cpp" line="48"/>
        <source>Ukey</source>
        <translation>Khóa bảo mật</translation>
    </message>
    <message>
        <location filename="../BiometricAuth/biometricdeviceinfo.cpp" line="50"/>
        <source>QRCode</source>
        <translation>QRCode</translation>
    </message>
</context>
<context>
    <name>DlgConnHidWifi</name>
    <message>
        <source>Connect to Hidden Wi-Fi Network</source>
        <translation type="vanished">连接到隐藏 Wi-Fi 网络</translation>
    </message>
    <message>
        <source>Add Hidden Wi-Fi</source>
        <translation type="vanished">加入隐藏Wi-Fi</translation>
    </message>
    <message>
        <source>Connection</source>
        <translation type="vanished">连接设置:</translation>
    </message>
    <message>
        <source>Wi-Fi name</source>
        <translation type="vanished">网络名称:</translation>
    </message>
    <message>
        <source>Wi-Fi security</source>
        <translation type="vanished">Wi-Fi 安全性:</translation>
    </message>
    <message>
        <source>Cancel</source>
        <translation type="vanished">取消</translation>
    </message>
    <message>
        <source>Connect</source>
        <translation type="vanished">连接</translation>
    </message>
    <message>
        <source>C_reate…</source>
        <translation type="vanished">新建...</translation>
    </message>
    <message>
        <source>None</source>
        <translation type="vanished">无</translation>
    </message>
    <message>
        <source>WPA &amp; WPA2 Personal</source>
        <translation type="vanished">WPA 及 WPA2 个人</translation>
    </message>
</context>
<context>
    <name>DlgConnHidWifiLeap</name>
    <message>
        <source>Connect to Hidden Wi-Fi Network</source>
        <translation type="vanished">连接到隐藏 Wi-Fi 网络</translation>
    </message>
    <message>
        <source>Add hidden Wi-Fi</source>
        <translation type="vanished">添加Wifi</translation>
    </message>
    <message>
        <source>Connection</source>
        <translation type="vanished">连接设置:</translation>
    </message>
    <message>
        <source>Network name</source>
        <translation type="vanished">网络名称:</translation>
    </message>
    <message>
        <source>Wi-Fi security</source>
        <translation type="vanished">Wi-Fi 安全性:</translation>
    </message>
    <message>
        <source>Username</source>
        <translation type="vanished">用户名</translation>
    </message>
    <message>
        <source>Password</source>
        <translation type="vanished">密码:</translation>
    </message>
    <message>
        <source>Cancel</source>
        <translation type="vanished">取消</translation>
    </message>
    <message>
        <source>Connect</source>
        <translation type="vanished">连接</translation>
    </message>
    <message>
        <source>C_reate…</source>
        <translation type="vanished">新建...</translation>
    </message>
    <message>
        <source>None</source>
        <translation type="vanished">无</translation>
    </message>
    <message>
        <source>WPA &amp; WPA2 Personal</source>
        <translation type="vanished">WPA 及 WPA2 个人</translation>
    </message>
    <message>
        <source>WEP 40/128-bit Key (Hex or ASCII)</source>
        <translation type="vanished">WEP 40/128位密钥(十六进制或ASCII码)</translation>
    </message>
    <message>
        <source>WEP 128-bit Passphrase</source>
        <translation type="vanished">WEP 128位密码</translation>
    </message>
    <message>
        <source>Dynamic WEP (802.1X)</source>
        <translation type="vanished">动态WEP(802.1X)</translation>
    </message>
</context>
<context>
    <name>DlgConnHidWifiSecFast</name>
    <message>
        <source>Connect to Hidden Wi-Fi Network</source>
        <translation type="vanished">连接到隐藏 Wi-Fi 网络</translation>
    </message>
    <message>
        <source>Add hidden Wi-Fi</source>
        <translation type="vanished">添加隐藏Wi-Fi</translation>
    </message>
    <message>
        <source>Connection</source>
        <translation type="vanished">连接设置:</translation>
    </message>
    <message>
        <source>Network name</source>
        <translation type="vanished">网络名称:</translation>
    </message>
    <message>
        <source>Wi-Fi security</source>
        <translation type="vanished">Wi-Fi 安全性:</translation>
    </message>
    <message>
        <source>Authentication</source>
        <translation type="vanished">身份验证</translation>
    </message>
    <message>
        <source>Anonymous identity</source>
        <translation type="vanished">匿名身份</translation>
    </message>
    <message>
        <source>PAC file</source>
        <translation type="vanished">PAC 文件</translation>
    </message>
    <message>
        <source>Inner authentication</source>
        <translation type="vanished">内部验证</translation>
    </message>
    <message>
        <source>Username</source>
        <translation type="vanished">用户名</translation>
    </message>
    <message>
        <source>Password</source>
        <translation type="vanished">密码:</translation>
    </message>
    <message>
        <source>Cancel</source>
        <translation type="vanished">取消</translation>
    </message>
    <message>
        <source>Connect</source>
        <translation type="vanished">连接</translation>
    </message>
    <message>
        <source>C_reate…</source>
        <translation type="vanished">新建...</translation>
    </message>
    <message>
        <source>None</source>
        <translation type="vanished">无</translation>
    </message>
    <message>
        <source>WPA &amp; WPA2 Personal</source>
        <translation type="vanished">WPA 及 WPA2 个人</translation>
    </message>
    <message>
        <source>WEP 40/128-bit Key (Hex or ASCII)</source>
        <translation type="vanished">WEP 40/128位密钥(十六进制或ASCII码)</translation>
    </message>
    <message>
        <source>WEP 128-bit Passphrase</source>
        <translation type="vanished">WEP 128位密码</translation>
    </message>
    <message>
        <source>Dynamic WEP (802.1X)</source>
        <translation type="vanished">动态WEP(802.1X)</translation>
    </message>
    <message>
        <source>Tunneled TLS</source>
        <translation type="vanished">隧道TLS</translation>
    </message>
    <message>
        <source>Protected EAP (PEAP)</source>
        <translation type="obsolete">保护EAP (PEAP)</translation>
    </message>
    <message>
        <source>Anonymous</source>
        <translation type="vanished">匿名</translation>
    </message>
    <message>
        <source>Authenticated</source>
        <translation type="vanished">已认证</translation>
    </message>
</context>
<context>
    <name>DlgConnHidWifiSecLeap</name>
    <message>
        <source>Connect to Hidden Wi-Fi Network</source>
        <translation type="vanished">连接到隐藏 Wi-Fi 网络</translation>
    </message>
    <message>
        <source>Add hidden Wi-Fi</source>
        <translation type="vanished">添加隐藏Wi-Fi</translation>
    </message>
    <message>
        <source>Connection</source>
        <translation type="vanished">连接设置:</translation>
    </message>
    <message>
        <source>Network name</source>
        <translation type="vanished">网络名称:</translation>
    </message>
    <message>
        <source>Wi-Fi security</source>
        <translation type="vanished">Wi-Fi 安全性:</translation>
    </message>
    <message>
        <source>Authentication</source>
        <translation type="vanished">身份验证</translation>
    </message>
    <message>
        <source>Username</source>
        <translation type="vanished">用户名</translation>
    </message>
    <message>
        <source>Password</source>
        <translation type="vanished">密码:</translation>
    </message>
    <message>
        <source>Cancel</source>
        <translation type="vanished">取消</translation>
    </message>
    <message>
        <source>Connect</source>
        <translation type="vanished">连接</translation>
    </message>
    <message>
        <source>C_reate…</source>
        <translation type="vanished">新建...</translation>
    </message>
    <message>
        <source>None</source>
        <translation type="vanished">无</translation>
    </message>
    <message>
        <source>WPA &amp; WPA2 Personal</source>
        <translation type="vanished">WPA 及 WPA2 个人</translation>
    </message>
    <message>
        <source>WEP 40/128-bit Key (Hex or ASCII)</source>
        <translation type="vanished">WEP 40/128位密钥(十六进制或ASCII码)</translation>
    </message>
    <message>
        <source>WEP 128-bit Passphrase</source>
        <translation type="vanished">WEP 128位密码</translation>
    </message>
    <message>
        <source>Dynamic WEP (802.1X)</source>
        <translation type="vanished">动态WEP(802.1X)</translation>
    </message>
    <message>
        <source>Tunneled TLS</source>
        <translation type="vanished">隧道TLS</translation>
    </message>
    <message>
        <source>Protected EAP (PEAP)</source>
        <translation type="vanished">保护EAP (PEAP)</translation>
    </message>
</context>
<context>
    <name>DlgConnHidWifiSecPeap</name>
    <message>
        <source>Connect to Hidden Wi-Fi Network</source>
        <translation type="vanished">连接到隐藏 Wi-Fi 网络</translation>
    </message>
    <message>
        <source>Add hidden Wi-Fi</source>
        <translation type="vanished">添加隐藏Wi-Fi</translation>
    </message>
    <message>
        <source>Connection</source>
        <translation type="vanished">连接设置:</translation>
    </message>
    <message>
        <source>Network name</source>
        <translation type="vanished">网络名称:</translation>
    </message>
    <message>
        <source>Wi-Fi security</source>
        <translation type="vanished">Wi-Fi 安全性:</translation>
    </message>
    <message>
        <source>Authentication</source>
        <translation type="vanished">身份认证</translation>
    </message>
    <message>
        <source>Anonymous identity</source>
        <translation type="vanished">匿名身份</translation>
    </message>
    <message>
        <source>Domain</source>
        <translation type="vanished">域</translation>
    </message>
    <message>
        <source>CA certificate</source>
        <translation type="vanished">CA证书</translation>
    </message>
    <message>
        <source>CA certificate password</source>
        <translation type="vanished">CA证书密码</translation>
    </message>
    <message>
        <source>No CA certificate is required</source>
        <translation type="vanished">不需要CA证书</translation>
    </message>
    <message>
        <source>PEAP version</source>
        <translation type="vanished">PEAP环境</translation>
    </message>
    <message>
        <source>Inner authentication</source>
        <translation type="vanished">内部验证</translation>
    </message>
    <message>
        <source>Username</source>
        <translation type="vanished">用户名</translation>
    </message>
    <message>
        <source>Password</source>
        <translation type="vanished">密码:</translation>
    </message>
    <message>
        <source>Cancel</source>
        <translation type="vanished">取消</translation>
    </message>
    <message>
        <source>Connect</source>
        <translation type="vanished">连接</translation>
    </message>
    <message>
        <source>None</source>
        <translation type="vanished">无</translation>
    </message>
    <message>
        <source>WPA &amp; WPA2 Personal</source>
        <translation type="vanished">WPA 及 WPA2 个人</translation>
    </message>
    <message>
        <source>WEP 40/128-bit Key (Hex or ASCII)</source>
        <translation type="vanished">WEP 40/128位密钥(十六进制或ASCII码)</translation>
    </message>
    <message>
        <source>WEP 128-bit Passphrase</source>
        <translation type="vanished">WEP 128位密码</translation>
    </message>
    <message>
        <source>Dynamic WEP (802.1X)</source>
        <translation type="vanished">动态WEP(802.1X)</translation>
    </message>
    <message>
        <source>Tunneled TLS</source>
        <translation type="vanished">隧道TLS</translation>
    </message>
    <message>
        <source>Protected EAP (PEAP)</source>
        <translation type="obsolete">保护EAP (PEAP)</translation>
    </message>
    <message>
        <source>Choose from file</source>
        <translation type="vanished">选择文件</translation>
    </message>
    <message>
        <source>Automatic</source>
        <translation type="vanished">自动</translation>
    </message>
</context>
<context>
    <name>DlgConnHidWifiSecPwd</name>
    <message>
        <source>Connect to Hidden Wi-Fi Network</source>
        <translation type="vanished">连接到隐藏 Wi-Fi 网络</translation>
    </message>
    <message>
        <source>Add hidden Wi-Fi</source>
        <translation type="vanished">添加Wifi</translation>
    </message>
    <message>
        <source>Connection</source>
        <translation type="vanished">连接设置:</translation>
    </message>
    <message>
        <source>Network name</source>
        <translation type="vanished">网络名称:</translation>
    </message>
    <message>
        <source>Wi-Fi security</source>
        <translation type="vanished">Wi-Fi 安全性:</translation>
    </message>
    <message>
        <source>Authentication</source>
        <translation type="vanished">认证</translation>
    </message>
    <message>
        <source>Username</source>
        <translation type="vanished">用户名</translation>
    </message>
    <message>
        <source>Password</source>
        <translation type="vanished">密码:</translation>
    </message>
    <message>
        <source>Cancel</source>
        <translation type="vanished">取消</translation>
    </message>
    <message>
        <source>Connect</source>
        <translation type="vanished">连接</translation>
    </message>
    <message>
        <source>C_reate…</source>
        <translation type="vanished">新建...</translation>
    </message>
    <message>
        <source>None</source>
        <translation type="vanished">无</translation>
    </message>
    <message>
        <source>WPA &amp; WPA2 Personal</source>
        <translation type="vanished">WPA 及 WPA2 个人</translation>
    </message>
    <message>
        <source>WEP 40/128-bit Key (Hex or ASCII)</source>
        <translation type="vanished">WEP 40/128位密钥(十六进制或ASCII码)</translation>
    </message>
    <message>
        <source>WEP 128-bit Passphrase</source>
        <translation type="vanished">WEP 128位密码</translation>
    </message>
    <message>
        <source>Dynamic WEP (802.1X)</source>
        <translation type="vanished">动态WEP(802.1X)</translation>
    </message>
    <message>
        <source>Tunneled TLS</source>
        <translation type="vanished">隧道TLS</translation>
    </message>
    <message>
        <source>Protected EAP (PEAP)</source>
        <translation type="obsolete">保护EAP (PEAP)</translation>
    </message>
</context>
<context>
    <name>DlgConnHidWifiSecTls</name>
    <message>
        <source>Connect to Hidden Wi-Fi Network</source>
        <translation type="vanished">连接到隐藏 Wi-Fi 网络</translation>
    </message>
    <message>
        <source>Add hidden Wi-Fi</source>
        <translation type="vanished">添加隐藏Wi-Fi</translation>
    </message>
    <message>
        <source>Connection</source>
        <translation type="vanished">连接设置:</translation>
    </message>
    <message>
        <source>Network name</source>
        <translation type="vanished">网络名称:</translation>
    </message>
    <message>
        <source>Wi-Fi security</source>
        <translation type="vanished">Wi-Fi 安全性:</translation>
    </message>
    <message>
        <source>Authentication</source>
        <translation type="vanished">认证</translation>
    </message>
    <message>
        <source>Identity</source>
        <translation type="vanished">身份</translation>
    </message>
    <message>
        <source>Domain</source>
        <translation type="vanished">域</translation>
    </message>
    <message>
        <source>CA certificate</source>
        <translation type="vanished">CA认证</translation>
    </message>
    <message>
        <source>CA certificate password</source>
        <translation type="vanished">CA认证密码</translation>
    </message>
    <message>
        <source>No CA certificate is required</source>
        <translation type="vanished">不需要CA认证</translation>
    </message>
    <message>
        <source>User certificate</source>
        <translation type="vanished">用户认证</translation>
    </message>
    <message>
        <source>User certificate password</source>
        <translation type="vanished">用户认证密码</translation>
    </message>
    <message>
        <source>User private key</source>
        <translation type="vanished">用户认证私钥</translation>
    </message>
    <message>
        <source>User key password</source>
        <translation type="vanished">用户密钥密码</translation>
    </message>
    <message>
        <source>Cancel</source>
        <translation type="vanished">取消</translation>
    </message>
    <message>
        <source>Connect</source>
        <translation type="vanished">连接</translation>
    </message>
    <message>
        <source>C_reate…</source>
        <translation type="vanished">新建...</translation>
    </message>
    <message>
        <source>None</source>
        <translation type="vanished">无</translation>
    </message>
    <message>
        <source>WPA &amp; WPA2 Personal</source>
        <translation type="vanished">WPA 及 WPA2 个人</translation>
    </message>
    <message>
        <source>WEP 40/128-bit Key (Hex or ASCII)</source>
        <translation type="obsolete">WEP 40/128位密钥(十六进制或ASCII码)</translation>
    </message>
    <message>
        <source>WEP 128-bit Passphrase</source>
        <translation type="obsolete">WEP 128位密码</translation>
    </message>
    <message>
        <source>Dynamic WEP (802.1X)</source>
        <translation type="obsolete">动态WEP(802.1X)</translation>
    </message>
    <message>
        <source>Tunneled TLS</source>
        <translation type="obsolete">隧道TLS</translation>
    </message>
    <message>
        <source>Protected EAP (PEAP)</source>
        <translation type="obsolete">保护EAP (PEAP)</translation>
    </message>
    <message>
        <source>Choose from file</source>
        <translation type="obsolete">选择文件</translation>
    </message>
</context>
<context>
    <name>DlgConnHidWifiSecTunnelTLS</name>
    <message>
        <source>Connect to Hidden Wi-Fi Network</source>
        <translation type="vanished">连接到隐藏 Wi-Fi 网络</translation>
    </message>
    <message>
        <source>Add hidden Wi-Fi</source>
        <translation type="vanished">添加Wifi</translation>
    </message>
    <message>
        <source>Connection</source>
        <translation type="vanished">连接设置:</translation>
    </message>
    <message>
        <source>Network name</source>
        <translation type="vanished">网络名称:</translation>
    </message>
    <message>
        <source>Wi-Fi security</source>
        <translation type="vanished">Wi-Fi 安全性:</translation>
    </message>
    <message>
        <source>Anonymous identity</source>
        <translation type="obsolete">匿名身份</translation>
    </message>
    <message>
        <source>Domain</source>
        <translation type="obsolete">域</translation>
    </message>
    <message>
        <source>Inner authentication</source>
        <translation type="obsolete">内部验证</translation>
    </message>
    <message>
        <source>Username</source>
        <translation type="vanished">用户名</translation>
    </message>
    <message>
        <source>Password</source>
        <translation type="vanished">密码:</translation>
    </message>
    <message>
        <source>Cancel</source>
        <translation type="vanished">取消</translation>
    </message>
    <message>
        <source>Connect</source>
        <translation type="vanished">连接</translation>
    </message>
    <message>
        <source>C_reate…</source>
        <translation type="vanished">新建...</translation>
    </message>
    <message>
        <source>None</source>
        <translation type="vanished">无</translation>
    </message>
    <message>
        <source>WPA &amp; WPA2 Personal</source>
        <translation type="vanished">WPA 及 WPA2 个人</translation>
    </message>
    <message>
        <source>WEP 40/128-bit Key (Hex or ASCII)</source>
        <translation type="obsolete">WEP 40/128位密钥(十六进制或ASCII码)</translation>
    </message>
    <message>
        <source>WEP 128-bit Passphrase</source>
        <translation type="obsolete">WEP 128位密码</translation>
    </message>
    <message>
        <source>Dynamic WEP (802.1X)</source>
        <translation type="obsolete">动态WEP(802.1X)</translation>
    </message>
    <message>
        <source>Tunneled TLS</source>
        <translation type="obsolete">隧道TLS</translation>
    </message>
    <message>
        <source>Protected EAP (PEAP)</source>
        <translation type="obsolete">保护EAP (PEAP)</translation>
    </message>
    <message>
        <source>Choose from file</source>
        <translation type="obsolete">选择文件</translation>
    </message>
</context>
<context>
    <name>DlgConnHidWifiWep</name>
    <message>
        <source>Connect to Hidden Wi-Fi Network</source>
        <translation type="vanished">连接到隐藏 Wi-Fi 网络</translation>
    </message>
    <message>
        <source>Connection</source>
        <translation type="vanished">连接设置:</translation>
    </message>
    <message>
        <source>Network name</source>
        <translation type="vanished">网络名称:</translation>
    </message>
    <message>
        <source>Wi-Fi security</source>
        <translation type="vanished">Wi-Fi 安全性:</translation>
    </message>
    <message>
        <source>Cancel</source>
        <translation type="vanished">取消</translation>
    </message>
    <message>
        <source>Connect</source>
        <translation type="vanished">连接</translation>
    </message>
    <message>
        <source>C_reate…</source>
        <translation type="vanished">新建...</translation>
    </message>
    <message>
        <source>None</source>
        <translation type="vanished">无</translation>
    </message>
    <message>
        <source>WPA &amp; WPA2 Personal</source>
        <translation type="vanished">WPA 及 WPA2 个人</translation>
    </message>
    <message>
        <source>WEP 40/128-bit Key (Hex or ASCII)</source>
        <translation type="obsolete">WEP 40/128位密钥(十六进制或ASCII码)</translation>
    </message>
    <message>
        <source>WEP 128-bit Passphrase</source>
        <translation type="obsolete">WEP 128位密码</translation>
    </message>
    <message>
        <source>Dynamic WEP (802.1X)</source>
        <translation type="obsolete">动态WEP(802.1X)</translation>
    </message>
</context>
<context>
    <name>DlgConnHidWifiWpa</name>
    <message>
        <source>Connect to Hidden Wi-Fi Network</source>
        <translation type="vanished">连接到隐藏 Wi-Fi 网络</translation>
    </message>
    <message>
        <source>Add Hidden Wi-Fi</source>
        <translation type="vanished">加入隐藏Wi-Fi</translation>
    </message>
    <message>
        <source>Connection</source>
        <translation type="vanished">连接设置:</translation>
    </message>
    <message>
        <source>Wi-Fi name</source>
        <translation type="vanished">网络名称:</translation>
    </message>
    <message>
        <source>Wi-Fi security</source>
        <translation type="vanished">Wi-Fi 安全性:</translation>
    </message>
    <message>
        <source>Password</source>
        <translation type="vanished">密码:</translation>
    </message>
    <message>
        <source>Cancel</source>
        <translation type="vanished">取消</translation>
    </message>
    <message>
        <source>Connect</source>
        <translation type="vanished">连接</translation>
    </message>
    <message>
        <source>C_reate…</source>
        <translation type="vanished">新建...</translation>
    </message>
    <message>
        <source>None</source>
        <translation type="vanished">无</translation>
    </message>
    <message>
        <source>WPA &amp; WPA2 Personal</source>
        <translation type="vanished">WPA 及 WPA2 个人</translation>
    </message>
</context>
<context>
    <name>DlgHotspotCreate</name>
    <message>
        <source>Create Hotspot</source>
        <translation type="vanished">创建个人热点</translation>
    </message>
    <message>
        <source>Network name</source>
        <translation type="vanished">网络名称:</translation>
    </message>
    <message>
        <source>Wi-Fi security</source>
        <translation type="vanished">Wi-Fi 安全性:</translation>
    </message>
    <message>
        <source>Password</source>
        <translation type="vanished">密码:</translation>
    </message>
    <message>
        <source>Cancel</source>
        <translation type="vanished">取消</translation>
    </message>
    <message>
        <source>Ok</source>
        <translation type="vanished">确定</translation>
    </message>
    <message>
        <source>None</source>
        <translation type="vanished">无</translation>
    </message>
    <message>
        <source>WPA &amp; WPA2 Personal</source>
        <translation type="vanished">WPA 及 WPA2 个人</translation>
    </message>
</context>
<context>
    <name>EngineDevice</name>
    <message>
        <location filename="../greeter/enginedevice.cpp" line="101"/>
        <source>yes</source>
        <translation>Chắc chắn</translation>
    </message>
    <message>
        <location filename="../greeter/enginedevice.cpp" line="101"/>
        <source>no</source>
        <translation>Không</translation>
    </message>
    <message>
        <location filename="../greeter/enginedevice.cpp" line="120"/>
        <source>Yes</source>
        <translation>Có</translation>
    </message>
    <message>
        <location filename="../greeter/enginedevice.cpp" line="120"/>
        <source>No</source>
        <translation>Không</translation>
    </message>
    <message>
        <location filename="../greeter/enginedevice.cpp" line="284"/>
        <source>%1% available, charged</source>
        <translation>%1% khả dụng, tính phí</translation>
    </message>
    <message>
        <location filename="../greeter/enginedevice.cpp" line="290"/>
        <source>Left %1h %2m (%3%)</source>
        <translation></translation>
    </message>
    <message>
        <location filename="../greeter/enginedevice.cpp" line="295"/>
        <source>%1% available</source>
        <translation></translation>
    </message>
    <message>
        <location filename="../greeter/enginedevice.cpp" line="301"/>
        <source>Left %1h %2m to full</source>
        <translation></translation>
    </message>
    <message>
        <location filename="../greeter/enginedevice.cpp" line="303"/>
        <source>charging (%1%)</source>
        <translation></translation>
    </message>
    <message>
        <location filename="../greeter/enginedevice.cpp" line="309"/>
        <source>%1 waiting to discharge (%2%)</source>
        <translation></translation>
    </message>
    <message>
        <location filename="../greeter/enginedevice.cpp" line="314"/>
        <source>%1 waiting to charge (%2%)</source>
        <translation></translation>
    </message>
    <message>
        <location filename="../greeter/enginedevice.cpp" line="334"/>
        <source>AC adapter</source>
        <translation></translation>
    </message>
    <message>
        <location filename="../greeter/enginedevice.cpp" line="338"/>
        <source>Laptop battery</source>
        <translation></translation>
    </message>
    <message>
        <location filename="../greeter/enginedevice.cpp" line="342"/>
        <source>UPS</source>
        <translation></translation>
    </message>
    <message>
        <location filename="../greeter/enginedevice.cpp" line="346"/>
        <source>Monitor</source>
        <translation></translation>
    </message>
    <message>
        <location filename="../greeter/enginedevice.cpp" line="350"/>
        <source>Mouse</source>
        <translation></translation>
    </message>
    <message>
        <location filename="../greeter/enginedevice.cpp" line="354"/>
        <source>Keyboard</source>
        <translation></translation>
    </message>
    <message>
        <location filename="../greeter/enginedevice.cpp" line="358"/>
        <source>PDA</source>
        <translation></translation>
    </message>
    <message>
        <location filename="../greeter/enginedevice.cpp" line="362"/>
        <source>Cell phone</source>
        <translation></translation>
    </message>
    <message>
        <location filename="../greeter/enginedevice.cpp" line="366"/>
        <source>Media player</source>
        <translation></translation>
    </message>
    <message>
        <location filename="../greeter/enginedevice.cpp" line="370"/>
        <source>Tablet</source>
        <translation></translation>
    </message>
    <message>
        <location filename="../greeter/enginedevice.cpp" line="374"/>
        <source>Computer</source>
        <translation></translation>
    </message>
    <message>
        <location filename="../greeter/enginedevice.cpp" line="378"/>
        <source>unrecognised</source>
        <translation></translation>
    </message>
</context>
<context>
    <name>FakeDialog</name>
    <message>
        <location filename="../common/fakedialog.cpp" line="63"/>
        <source>cancel</source>
        <translation>hủy</translation>
    </message>
</context>
<context>
    <name>GreeterWindow</name>
    <message>
        <source>Power dialog</source>
        <translation type="vanished">电源对话框</translation>
    </message>
    <message>
        <source>On-screen keyboard, providing virtual keyboard function</source>
        <translation type="vanished">屏幕键盘，提供虚拟键盘功能</translation>
    </message>
    <message>
        <source>Set the desktop environment for the selected user to log in.If the user is logged in, it will take effect after logging in again</source>
        <translation type="vanished">设置选中用户登录后的桌面环境，如果用户已经登录，则会在重新登录后生效</translation>
    </message>
    <message>
        <source>Set the language of the selected user after logging in. If the user is logged in, it will take effect after logging in again.</source>
        <translation type="vanished">设置选中用户登录后的语言，如果用户已登录，则在重新登录后生效</translation>
    </message>
    <message>
        <location filename="../greeter/greeterwindow.cpp" line="167"/>
        <source>LAN</source>
        <translation>Mạng có dây</translation>
    </message>
    <message>
        <location filename="../greeter/greeterwindow.cpp" line="169"/>
        <source>WLAN</source>
        <translation>Mạng cục bộ không dây</translation>
    </message>
    <message>
        <location filename="../greeter/greeterwindow.cpp" line="385"/>
        <source>VirtualKeyboard</source>
        <translation>Bàn phím ảo</translation>
    </message>
    <message>
        <location filename="../greeter/greeterwindow.cpp" line="387"/>
        <source>Power</source>
        <translation>Nguồn điện</translation>
    </message>
    <message>
        <location filename="../greeter/greeterwindow.cpp" line="389"/>
        <source>SwitchUser</source>
        <translation>Người dùng chuyển đổi</translation>
    </message>
    <message>
        <location filename="../greeter/greeterwindow.cpp" line="391"/>
        <source>PowerInfo</source>
        <translation>Thông tin điện</translation>
    </message>
    <message>
        <location filename="../greeter/greeterwindow.cpp" line="970"/>
        <source>Shut Down</source>
        <translation>Tắt máy</translation>
    </message>
    <message>
        <location filename="../greeter/greeterwindow.cpp" line="973"/>
        <source>Restart</source>
        <translation>Khởi động lại</translation>
    </message>
    <message>
        <location filename="../greeter/greeterwindow.cpp" line="976"/>
        <source>Sleep</source>
        <translation>Chế độ ngủ</translation>
    </message>
    <message>
        <location filename="../greeter/greeterwindow.cpp" line="979"/>
        <source>Hibernate</source>
        <translation>Ngủ đông</translation>
    </message>
    <message>
        <location filename="../greeter/greeterwindow.cpp" line="1578"/>
        <location filename="../greeter/greeterwindow.cpp" line="2171"/>
        <source>Login</source>
        <translation>Đăng nhập</translation>
    </message>
</context>
<context>
    <name>GreeterWrapper</name>
    <message>
        <location filename="../greeter/greeterwrapper.cpp" line="171"/>
        <source>failed to start session.</source>
        <translation>Khởi động phiên làm việc thất bại.</translation>
    </message>
</context>
<context>
    <name>IconEdit</name>
    <message>
        <location filename="../greeter/iconedit.cpp" line="137"/>
        <source>OK</source>
        <translation>Chắc chắc</translation>
    </message>
    <message>
        <location filename="../greeter/iconedit.cpp" line="270"/>
        <location filename="../greeter/iconedit.cpp" line="318"/>
        <source>Password: </source>
        <translation>Mật khẩu: </translation>
    </message>
    <message>
        <location filename="../greeter/iconedit.cpp" line="272"/>
        <location filename="../greeter/iconedit.cpp" line="320"/>
        <source>Username</source>
        <translation>Tên người dùng</translation>
    </message>
</context>
<context>
    <name>KylinNM</name>
    <message>
        <source>Ethernet Networks</source>
        <translation type="vanished">可用有线网络列表</translation>
    </message>
    <message>
        <source>New LAN</source>
        <translation type="vanished">新建有线网络</translation>
    </message>
    <message>
        <source>Wifi Networks</source>
        <translation type="vanished">加入其他网络</translation>
    </message>
    <message>
        <source>Hide WiFi</source>
        <translation type="vanished">加入其他网络</translation>
    </message>
    <message>
        <source>No usable network in the list</source>
        <translation type="vanished">列表暂无可连接网络</translation>
    </message>
    <message>
        <source>Ethernet</source>
        <translation type="vanished">有线网络</translation>
    </message>
    <message>
        <source>有线网络</source>
        <translation type="vanished">有线网络</translation>
    </message>
    <message>
        <source>Wifi</source>
        <translation type="vanished">Wifi</translation>
    </message>
    <message>
        <source>无线网络</source>
        <translation type="vanished">无线网络</translation>
    </message>
    <message>
        <source>HotSpot</source>
        <translation type="vanished">个人热点</translation>
    </message>
    <message>
        <source>FlyMode</source>
        <translation type="vanished">飞行模式</translation>
    </message>
    <message>
        <source>Advanced</source>
        <translation type="vanished">设置网络</translation>
    </message>
    <message>
        <source>Show KylinNM</source>
        <translation type="vanished">Show KylinNM</translation>
    </message>
    <message>
        <source>Inactivated LAN</source>
        <translation type="vanished">未激活</translation>
    </message>
    <message>
        <source>Other WLAN</source>
        <translation type="vanished">其他</translation>
    </message>
    <message>
        <source>LAN</source>
        <translation type="vanished">有线网络</translation>
    </message>
    <message>
        <source>WLAN</source>
        <translation type="vanished">无线局域网</translation>
    </message>
    <message>
        <source>No wireless card detected</source>
        <translation type="vanished">未检测到无线网卡</translation>
    </message>
    <message>
        <source>Activated LAN</source>
        <translation type="vanished">已激活</translation>
    </message>
    <message>
        <source>Activated WLAN</source>
        <translation type="vanished">已激活</translation>
    </message>
    <message>
        <source>Not connected</source>
        <translation type="vanished">当前未连接任何网络</translation>
    </message>
    <message>
        <source>Disconnected</source>
        <translation type="vanished">未连接</translation>
    </message>
    <message>
        <source>NetOn,</source>
        <translation type="vanished">网络开启</translation>
    </message>
    <message>
        <source>No Other Wired Network Scheme</source>
        <translation type="vanished">列表中无其他有线网络</translation>
    </message>
    <message>
        <source>No Other Wireless Network Scheme</source>
        <translation type="vanished">未检测到其他无线网络</translation>
    </message>
    <message>
        <source>Wired net is disconnected</source>
        <translation type="vanished">断开有线网络</translation>
    </message>
    <message>
        <source>Wi-Fi is disconnected</source>
        <translation type="vanished">断开无线网络</translation>
    </message>
    <message>
        <source>Conn Ethernet Success</source>
        <translation type="vanished">连接有线网络成功</translation>
    </message>
    <message>
        <source>Conn Ethernet Fail</source>
        <translation type="vanished">连接有线网络失败</translation>
    </message>
    <message>
        <source>Conn Wifi Success</source>
        <translation type="vanished">连接无线网络成功</translation>
    </message>
    <message>
        <source>Confirm your Wi-Fi password or usable of wireless card</source>
        <translation type="vanished">请确认Wi-Fi密码或无线设备</translation>
    </message>
</context>
<context>
    <name>LettersWidget</name>
    <message>
        <source>Symbol</source>
        <translation type="vanished">符号</translation>
    </message>
    <message>
        <source>Num</source>
        <translation type="vanished">数字</translation>
    </message>
    <message>
        <location filename="../VirtualKeyboard/src/letterswidget.cpp" line="140"/>
        <source>&amp;&amp;?!</source>
        <translation>&amp;&amp;?!</translation>
    </message>
    <message>
        <location filename="../VirtualKeyboard/src/letterswidget.cpp" line="154"/>
        <source>123</source>
        <translation>123</translation>
    </message>
</context>
<context>
    <name>LoginOptionsWidget</name>
    <message>
        <location filename="../greeter/loginoptionswidget.cpp" line="76"/>
        <location filename="../greeter/loginoptionswidget.cpp" line="99"/>
        <source>Login Options</source>
        <translation>Tùy chọn đăng nhập</translation>
    </message>
    <message>
        <location filename="../greeter/loginoptionswidget.cpp" line="330"/>
        <source>Password</source>
        <translation>Mật khẩu</translation>
    </message>
    <message>
        <location filename="../greeter/loginoptionswidget.cpp" line="356"/>
        <source>Other</source>
        <translation>Khác</translation>
    </message>
    <message>
        <source>Wechat</source>
        <translation type="vanished">微信</translation>
    </message>
    <message>
        <location filename="../greeter/loginoptionswidget.cpp" line="766"/>
        <source>Identify device removed!</source>
        <translation>Thiết bị xác thực đã bị gỡ bỏ!</translation>
    </message>
</context>
<context>
    <name>LoginWindow</name>
    <message>
        <source>logged in</source>
        <translation type="vanished">已登录</translation>
    </message>
    <message>
        <location filename="../greeter/loginwindow.cpp" line="1030"/>
        <source>login</source>
        <translation>Đăng nhập</translation>
    </message>
    <message>
        <source>Incorrect user name, please input again</source>
        <translation type="vanished">用户名不正确，请重新输入</translation>
    </message>
    <message>
        <source>Biometric Authentication</source>
        <translation type="vanished">生物识别认证</translation>
    </message>
    <message>
        <source>Password Authentication</source>
        <translation type="vanished">密码认证</translation>
    </message>
    <message>
        <source>Other Devices</source>
        <translation type="vanished">其他设备</translation>
    </message>
    <message>
        <location filename="../greeter/loginwindow.cpp" line="1444"/>
        <source>Retry</source>
        <translation>Thử lại</translation>
    </message>
    <message>
        <source>Please enter your password or enroll your fingerprint </source>
        <translation type="vanished">请输入密码或者录入指纹</translation>
    </message>
    <message>
        <location filename="../greeter/loginwindow.cpp" line="1243"/>
        <source>Password: </source>
        <translation>Mật khẩu: </translation>
    </message>
    <message>
        <location filename="../greeter/loginwindow.cpp" line="1386"/>
        <source>User name input error!</source>
        <translation>Lỗi nhập tên người dùng!</translation>
    </message>
    <message>
        <location filename="../greeter/loginwindow.cpp" line="1389"/>
        <location filename="../greeter/loginwindow.cpp" line="1394"/>
        <source>Authentication failure, Please try again</source>
        <translation>Xác thực không thành công, Vui lòng thử lại</translation>
    </message>
    <message>
        <source>Biometric/code scanning authentication failed too many times, please enter the password.</source>
        <translation type="vanished">生物/扫码验证失败达最大次数，请使用密码解锁</translation>
    </message>
    <message>
        <source>Biometric/code scan authentication failed too many times, please enter the password.</source>
        <translation type="vanished">生物/扫码验证失败达最大次数，请使用密码解锁</translation>
    </message>
    <message>
        <source>Bioauth/code scan authentication failed, you still have %1 verification opportunities</source>
        <translation type="vanished">生物/扫码验证失败，您还有%1次尝试机会</translation>
    </message>
    <message>
        <source>NET Exception</source>
        <translation type="vanished">网络异常</translation>
    </message>
    <message>
        <location filename="../greeter/loginwindow.cpp" line="2155"/>
        <location filename="../greeter/loginwindow.cpp" line="2156"/>
        <location filename="../greeter/loginwindow.cpp" line="2233"/>
        <location filename="../greeter/loginwindow.cpp" line="2234"/>
        <source>Please try again in %1 minutes.</source>
        <translation>Vui lòng thử lại sau %1 phút.</translation>
    </message>
    <message>
        <location filename="../greeter/loginwindow.cpp" line="2165"/>
        <location filename="../greeter/loginwindow.cpp" line="2166"/>
        <location filename="../greeter/loginwindow.cpp" line="2244"/>
        <location filename="../greeter/loginwindow.cpp" line="2245"/>
        <source>Please try again in %1 seconds.</source>
        <translation>Vui lòng thử lại sau%1 giây.</translation>
    </message>
    <message>
        <location filename="../greeter/loginwindow.cpp" line="2175"/>
        <location filename="../greeter/loginwindow.cpp" line="2176"/>
        <location filename="../greeter/loginwindow.cpp" line="2254"/>
        <location filename="../greeter/loginwindow.cpp" line="2255"/>
        <source>Account locked permanently.</source>
        <translation>Tài khoản đã bị khóa vĩnh viễn.</translation>
    </message>
    <message>
        <location filename="../greeter/loginwindow.cpp" line="1312"/>
        <source>Password cannot be empty</source>
        <translation>Mật khẩu không được trống</translation>
    </message>
    <message>
        <location filename="../greeter/loginwindow.cpp" line="202"/>
        <location filename="../greeter/loginwindow.cpp" line="324"/>
        <source>Enter the ukey password</source>
        <translation>Nhập mật khẩu UKey</translation>
    </message>
    <message>
        <location filename="../greeter/loginwindow.cpp" line="221"/>
        <location filename="../greeter/loginwindow.cpp" line="328"/>
        <source>Insert the ukey into the USB port</source>
        <translation>Cắm ukey vào cổng USB</translation>
    </message>
    <message>
        <location filename="../greeter/loginwindow.cpp" line="538"/>
        <source>Verify face recognition or enter password to unlock</source>
        <translation>Xác thực nhận dạng khuôn mặt hoặc nhập mật khẩu để mở khóa</translation>
    </message>
    <message>
        <location filename="../greeter/loginwindow.cpp" line="543"/>
        <source>Press fingerprint or enter password to unlock</source>
        <translation>Nhấn dấu vân tay hoặc nhập mật khẩu để mở khóa</translation>
    </message>
    <message>
        <location filename="../greeter/loginwindow.cpp" line="548"/>
        <source>Verify voiceprint or enter password to unlock</source>
        <translation>Xác minh dấu thoại hoặc nhập mật khẩu để mở khóa</translation>
    </message>
    <message>
        <location filename="../greeter/loginwindow.cpp" line="553"/>
        <source>Verify finger vein or enter password to unlock</source>
        <translation>Xác minh tĩnh mạch ngón tay hoặc nhập mật khẩu để mở khóa</translation>
    </message>
    <message>
        <location filename="../greeter/loginwindow.cpp" line="558"/>
        <source>Verify iris or enter password to unlock</source>
        <translation>Xác minh mống mắt hoặc nhập mật khẩu để mở khóa</translation>
    </message>
    <message>
        <location filename="../greeter/loginwindow.cpp" line="1247"/>
        <source>Input Password</source>
        <translation>Nhập mật khẩu</translation>
    </message>
    <message>
        <location filename="../greeter/loginwindow.cpp" line="1732"/>
        <location filename="../greeter/loginwindow.cpp" line="1894"/>
        <location filename="../greeter/loginwindow.cpp" line="2036"/>
        <location filename="../greeter/loginwindow.cpp" line="2397"/>
        <source>Failed to verify %1, please enter password to unlock</source>
        <translation>Không xác minh được %1, vui lòng nhập mật khẩu để mở khóa</translation>
    </message>
    <message>
        <location filename="../greeter/loginwindow.cpp" line="1734"/>
        <location filename="../greeter/loginwindow.cpp" line="1896"/>
        <location filename="../greeter/loginwindow.cpp" line="2040"/>
        <location filename="../greeter/loginwindow.cpp" line="2042"/>
        <location filename="../greeter/loginwindow.cpp" line="2399"/>
        <source>Unable to verify %1, please enter password to unlock</source>
        <translation>Không thể xác minh%1, vui lòng nhập mật khẩu để mở khóa</translation>
    </message>
    <message>
        <location filename="../greeter/loginwindow.cpp" line="2071"/>
        <source>Abnormal network</source>
        <translation>Lỗi mạng</translation>
    </message>
    <message>
        <source>Use the bound wechat scanning code or enter the password to log in</source>
        <translation type="vanished">使用绑定的微信扫码或输入密码登录</translation>
    </message>
    <message>
        <source>Failed to verify %1, please enter password.</source>
        <translation type="vanished">验证%1失败，请输入密码.</translation>
    </message>
    <message>
        <source>Unable to verify %1, please enter password.</source>
        <translation type="vanished">无法验证%1，请输入密码.</translation>
    </message>
    <message>
        <location filename="../greeter/loginwindow.cpp" line="2056"/>
        <location filename="../greeter/loginwindow.cpp" line="2060"/>
        <source>Failed to verify %1, you still have %2 verification opportunities</source>
        <translation>Xác thực %1 thất bại,bạn còn %2 lần thử</translation>
    </message>
    <message>
        <source>Incorrect password, please input again</source>
        <translation type="obsolete">密码错误，请重新输入</translation>
    </message>
    <message>
        <location filename="../greeter/loginwindow.cpp" line="320"/>
        <location filename="../greeter/loginwindow.cpp" line="783"/>
        <location filename="../greeter/loginwindow.cpp" line="1007"/>
        <location filename="../greeter/loginwindow.cpp" line="1008"/>
        <location filename="../greeter/loginwindow.cpp" line="1435"/>
        <source>Login</source>
        <translation>Đăng nhập</translation>
    </message>
    <message>
        <location filename="../greeter/loginwindow.cpp" line="563"/>
        <source>Use the bound wechat scanning code or enter the password to unlock</source>
        <translation>Vui lòng quét mã QR WeChat đã liên kết hoặc nhập mật khẩu để mở khóa</translation>
    </message>
    <message>
        <location filename="../greeter/loginwindow.cpp" line="1252"/>
        <source>Username</source>
        <translation>Tên người dùng</translation>
    </message>
</context>
<context>
    <name>MainWindow</name>
    <message>
        <location filename="../display-switch/ui_mainwindow.h" line="127"/>
        <source>MainWindow</source>
        <translation>Cửa sổ chính</translation>
    </message>
</context>
<context>
    <name>NumbersWidget</name>
    <message>
        <source>Symbol</source>
        <translation type="vanished">符号</translation>
    </message>
    <message>
        <location filename="../VirtualKeyboard/src/numberswidget.cpp" line="143"/>
        <source>&amp;&amp;?!</source>
        <translation>&amp;&amp;?!</translation>
    </message>
    <message>
        <location filename="../VirtualKeyboard/src/numberswidget.cpp" line="157"/>
        <source>Return</source>
        <translation>Quay lại</translation>
    </message>
</context>
<context>
    <name>OneConnForm</name>
    <message>
        <source>Automatically join the network</source>
        <translation type="obsolete">自动加入该网络</translation>
    </message>
    <message>
        <source>Connect</source>
        <translation type="vanished">连接</translation>
    </message>
    <message>
        <source>Disconnect</source>
        <translation type="vanished">断开连接</translation>
    </message>
    <message>
        <source>Input Password...</source>
        <translation type="vanished">输入密码...</translation>
    </message>
    <message>
        <source>自动连接</source>
        <translation type="vanished">自动连接</translation>
    </message>
    <message>
        <source>Connect to Hidden Wi-Fi Network</source>
        <translation type="vanished">连接到隐藏 Wi-Fi 网络</translation>
    </message>
    <message>
        <source>Rate</source>
        <translation type="obsolete">速率</translation>
    </message>
    <message>
        <source>None</source>
        <translation type="vanished">无</translation>
    </message>
    <message>
        <source>WiFi Security：</source>
        <translation type="vanished">WiFi安全性：</translation>
    </message>
    <message>
        <source>MAC：</source>
        <translation type="vanished">物理地址：</translation>
    </message>
    <message>
        <source>Conn Wifi Failed</source>
        <translation type="vanished">连接无线网络失败</translation>
    </message>
</context>
<context>
    <name>OneLancForm</name>
    <message>
        <source>Connect</source>
        <translation type="vanished">连接</translation>
    </message>
    <message>
        <source>Disconnect</source>
        <translation type="vanished">断开连接</translation>
    </message>
    <message>
        <source>No Configuration</source>
        <translation type="vanished">未配置</translation>
    </message>
    <message>
        <source>IPv4：</source>
        <translation type="vanished">IPv4地址：</translation>
    </message>
    <message>
        <source>IPv6：</source>
        <translation type="vanished">IPv6地址：</translation>
    </message>
    <message>
        <source>BandWidth：</source>
        <translation type="vanished">带宽：</translation>
    </message>
    <message>
        <source>MAC：</source>
        <translation type="vanished">物理地址：</translation>
    </message>
</context>
<context>
    <name>PowerManager</name>
    <message>
        <location filename="../greeter/powerwindow.cpp" line="130"/>
        <source>Close all apps, turn off your computer, and then turn your computer back on</source>
        <translation>Đóng tất cả các ứng dụng, tắt máy tính, sau đó mở lại máy tính</translation>
    </message>
    <message>
        <location filename="../greeter/powerwindow.cpp" line="132"/>
        <source>Close all apps, and then shut down your computer</source>
        <translation>Đóng tất cả các ứng dụng, sau đó tắt máy tính của bạn</translation>
    </message>
    <message>
        <location filename="../greeter/powerwindow.cpp" line="134"/>
        <source>The computer stays on, but consumes less power. The app stays open and can quickly wake up and revert to where you left off</source>
        <translation>Máy tính vẫn hoạt động, nhưng tiêu thụ ít điện năng hơn. Ứng dụng vẫn mở và có thể nhanh chóng đánh thức và hoàn nguyên về nơi bạn đã dừng lại</translation>
    </message>
    <message>
        <location filename="../greeter/powerwindow.cpp" line="137"/>
        <source>Turn off your computer, but the app stays open. When the computer is turned on, it can be restored to the state you left</source>
        <translation>Tắt máy tính của bạn nhưng ứng dụng vẫn mở. Khi máy tính được bật, nó có thể được khôi phục về trạng thái bạn đã để lại</translation>
    </message>
    <message>
        <location filename="../greeter/powerwindow.cpp" line="396"/>
        <source>Switch User</source>
        <translation>Chuyển đổi người dùng</translation>
    </message>
    <message>
        <source>Reboot</source>
        <translation type="vanished">重启</translation>
    </message>
    <message>
        <location filename="../greeter/powerwindow.cpp" line="473"/>
        <source>Shut Down</source>
        <translation>Tắt máy</translation>
    </message>
    <message>
        <location filename="../greeter/powerwindow.cpp" line="494"/>
        <source>Hibernate</source>
        <translation>Ngủ đông</translation>
    </message>
    <message>
        <location filename="../greeter/powerwindow.cpp" line="517"/>
        <source>Suspend</source>
        <translation>Đình chỉ</translation>
    </message>
    <message>
        <source>Sleep</source>
        <translation type="vanished">休眠</translation>
    </message>
    <message>
        <location filename="../greeter/powerwindow.cpp" line="397"/>
        <location filename="../greeter/powerwindow.cpp" line="453"/>
        <source>Restart</source>
        <translation>Khởi động lại</translation>
    </message>
    <message>
        <location filename="../greeter/powerwindow.cpp" line="398"/>
        <source>Power Off</source>
        <translation>Tắt nguồn</translation>
    </message>
</context>
<context>
    <name>S:</name>
    <message>
        <location filename="../greeter/enginedevice.cpp" line="308"/>
        <source></source>
        <comment>tablet device</comment>
        <translation></translation>
    </message>
</context>
<context>
    <name>Surewidget</name>
    <message>
        <location filename="../greeter/surewidget.ui" line="14"/>
        <source>Form</source>
        <translation>Mẫu</translation>
    </message>
    <message>
        <location filename="../greeter/surewidget.ui" line="56"/>
        <location filename="../greeter/surewidget.ui" line="66"/>
        <location filename="../greeter/surewidget.ui" line="79"/>
        <source>TextLabel</source>
        <translation>Hộp văn bản</translation>
    </message>
    <message>
        <location filename="../greeter/surewidget.ui" line="130"/>
        <source>Cancel</source>
        <translation>Hủy</translation>
    </message>
    <message>
        <location filename="../greeter/surewidget.ui" line="143"/>
        <source>Confirm</source>
        <translation>Xác nhận</translation>
    </message>
    <message>
        <location filename="../greeter/surewidget.cpp" line="21"/>
        <location filename="../greeter/surewidget.cpp" line="75"/>
        <source>Multiple users are logged in at the same time.Are you sure you want to reboot this system?</source>
        <translation>Nhiều người dùng được đăng nhập cùng một lúc. Bạn có chắc chắn muốn khởi động lại hệ thống này không?</translation>
    </message>
    <message>
        <location filename="../greeter/surewidget.cpp" line="40"/>
        <location filename="../greeter/surewidget.cpp" line="68"/>
        <source>There&apos;s a program running, this process cannot be interrupted.</source>
        <translation>Chương trình đang chạy, quá trình này không thể bị gián đoạn.</translation>
    </message>
    <message>
        <location filename="../greeter/surewidget.cpp" line="43"/>
        <source>The system will automatically %1 after the program is executed. Do not shut down the computer.</source>
        <translation>Hệ thống sẽ tự động %1 sau khi hoàn tất cài đặt, vui lòng không tắt máy tính.</translation>
    </message>
    <message>
        <location filename="../greeter/surewidget.cpp" line="72"/>
        <source>Click OK and the system will automatically %1 after the program is executed.</source>
        <translation>Nhấp vào OK và hệ thống sẽ tự động %1 sau khi chương trình được thực thi.</translation>
    </message>
    <message>
        <location filename="../greeter/surewidget.cpp" line="111"/>
        <source>Shut Down</source>
        <translation>Tắt máy</translation>
    </message>
    <message>
        <location filename="../greeter/surewidget.cpp" line="114"/>
        <source>Restart</source>
        <translation>Khởi động lại</translation>
    </message>
    <message>
        <location filename="../greeter/surewidget.cpp" line="117"/>
        <source>Sleep</source>
        <translation>Ngủ</translation>
    </message>
    <message>
        <location filename="../greeter/surewidget.cpp" line="120"/>
        <source>Hibernate</source>
        <translation>Ngủ đông</translation>
    </message>
</context>
<context>
    <name>TouchScreenSet</name>
    <message>
        <location filename="../common/touchscreenset.cpp" line="83"/>
        <source>%1</source>
        <translation>%1</translation>
    </message>
</context>
<context>
    <name>UsersModel</name>
    <message>
        <location filename="../greeter/usersmodel.cpp" line="58"/>
        <source>Guest Session</source>
        <translation>Đăng nhập khách</translation>
    </message>
    <message>
        <location filename="../greeter/usersmodel.cpp" line="48"/>
        <location filename="../greeter/usersmodel.cpp" line="50"/>
        <source>Guest</source>
        <translation>Khách</translation>
    </message>
    <message>
        <location filename="../greeter/usersmodel.cpp" line="76"/>
        <location filename="../greeter/usersmodel.cpp" line="78"/>
        <location filename="../greeter/usersmodel.cpp" line="85"/>
        <location filename="../greeter/usersmodel.cpp" line="100"/>
        <source>Login</source>
        <translation>Đăng nhập</translation>
    </message>
</context>
<context>
    <name>Utils</name>
    <message>
        <source>kylin network applet desktop message</source>
        <translation type="vanished">麒麟网络小程序桌面消息</translation>
    </message>
</context>
</TS>
