/*
 * Copyright (C) 2023, KylinSoft Co., Ltd.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, see <http://www.gnu.org/licenses/>.
 *
**/
#include <QCoreApplication>
#include <QTimer>
#include <QDebug>
#include <QObject>

class GreeterHelp: public QObject
{
    Q_OBJECT
    Q_CLASSINFO("D-Bus Interface", "org.greeter.nm")
public:
    GreeterHelp();
public slots:
    bool execConnWifiPWD(const QString connName,const QString password);
    void execConnWifiHiden(const QString connName,const QString password);
    void exitGreeterHelp();
    void nmNetworkSwitch(bool val);
    void nmWifiSwitch(bool val);
    void nmConnAdd(QString conn,QString ifname);
    void nmConnDelete(QString conn);
    void kylin_network_set_manualall(QString con_name, QString addr, QString mask, QString gateway, QString dns);
    void kylin_network_del_ethernet_con(QString con_name);
    void kylin_network_set_automethod(QString con_name);
    void kylin_network_set_con_down(QString con_name);
    bool kylin_network_set_autoconnect(QString con_name,bool autocon);
    bool kylin_network_get_autoconnect(QString con_name);
signals:
};
