/*
 * Copyright (C) 2021 KylinSoftCo., Ltd.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, see <http://www.gnu.org/licenses/>.
 *
**/
#ifndef BATTERYWIDGET_H
#define BATTERYWIDGET_H

#include <QObject>
#include <QWidget>
#include <QLabel>
#include <QDBusInterface>
#include <QPushButton>
#include <QPoint>
#include "enginedevice.h"

#define FREEDESKTOP_UPOWER                  "org.freedesktop.DBus.Properties"
#define UPOWER_INTERFACE                    "org.freedesktop.UPower"
#define UPOWER_PATH                         "/org/freedesktop/UPower"
#define UPOWER_SERVICE                      "org.freedesktop.UPower"
#define UPOWER_DISPLAY_PATH                 "/org/freedesktop/UPower/devices/DisplayDevice"
#define UPOWER_DIVICES_SERVICE              "org.freedesktop.UPower.Device"

class BatteryWidget : public QWidget
{
    Q_OBJECT
public:
    BatteryWidget(QPoint point, QWidget *parent = nullptr);

    void initUi();
    void setupComponent();
    void setPoint(QPoint point);
    QString getBatteryIconName();

protected:
    void showEvent(QShowEvent *event);
    void paintEvent(QPaintEvent *event);

private:
    QLabel *mModeLabel;
    QPushButton *mIconBtn;
    QLabel *mValueLabel;
    QLabel *mStatusLabel;
    QPoint mPoint;

    QDBusInterface *batInterface;
    QDBusInterface *iface;
    QDBusInterface *dface = nullptr;

    EngineDevice* ed;

private Q_SLOTS:
    void setBatteryIcon(QString icon);
    void onBatteryChanged(QStringList args);
    void dealMessage(QDBusMessage);

};

#endif // BATTERYWIDGET_H
