<?xml version="1.0" encoding="utf-8"?>
<!--
   Copyright (C) 2023 KylinSoft Co., Ltd.

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 3, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, see &lt;http://www.gnu.org/licenses/&gt;.
 -->
<!DOCTYPE TS>
<TS version="2.1" language="ru_RU">
<context>
    <name>AgreementWindow</name>
    <message>
        <location filename="../greeter/agreementwindow.cpp" line="32"/>
        <source>I know</source>
        <translation type="unfinished"></translation>
    </message>
</context>
<context>
    <name>BatteryWidget</name>
    <message>
        <location filename="../greeter/batterywidget.cpp" line="164"/>
        <source>Charging...</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../greeter/batterywidget.cpp" line="166"/>
        <source>fully charged</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../greeter/batterywidget.cpp" line="170"/>
        <source>PowerMode</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../greeter/batterywidget.cpp" line="173"/>
        <source>BatteryMode</source>
        <translation type="unfinished"></translation>
    </message>
</context>
<context>
    <name>CharsMoreWidget</name>
    <message>
        <location filename="../VirtualKeyboard/src/charsmorewidget.cpp" line="166"/>
        <source>&amp;&amp;?!</source>
        <translation type="unfinished"></translation>
    </message>
</context>
<context>
    <name>CharsWidget</name>
    <message>
        <location filename="../VirtualKeyboard/src/charswidget.cpp" line="97"/>
        <source>More</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../VirtualKeyboard/src/charswidget.cpp" line="111"/>
        <source>ABC</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../VirtualKeyboard/src/charswidget.cpp" line="124"/>
        <source>123</source>
        <translation type="unfinished"></translation>
    </message>
</context>
<context>
    <name>DeviceType</name>
    <message>
        <location filename="../BiometricAuth/biometricdeviceinfo.cpp" line="38"/>
        <source>FingerPrint</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../BiometricAuth/biometricdeviceinfo.cpp" line="40"/>
        <source>FingerVein</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../BiometricAuth/biometricdeviceinfo.cpp" line="42"/>
        <source>Iris</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../BiometricAuth/biometricdeviceinfo.cpp" line="44"/>
        <source>Face</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../BiometricAuth/biometricdeviceinfo.cpp" line="46"/>
        <source>VoicePrint</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../BiometricAuth/biometricdeviceinfo.cpp" line="48"/>
        <source>Ukey</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../BiometricAuth/biometricdeviceinfo.cpp" line="50"/>
        <source>QRCode</source>
        <translation type="unfinished"></translation>
    </message>
</context>
<context>
    <name>EngineDevice</name>
    <message>
        <location filename="../greeter/enginedevice.cpp" line="101"/>
        <source>yes</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../greeter/enginedevice.cpp" line="101"/>
        <source>no</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../greeter/enginedevice.cpp" line="120"/>
        <source>Yes</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../greeter/enginedevice.cpp" line="120"/>
        <source>No</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../greeter/enginedevice.cpp" line="284"/>
        <source>%1% available, charged</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../greeter/enginedevice.cpp" line="290"/>
        <source>Left %1h %2m (%3%)</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../greeter/enginedevice.cpp" line="295"/>
        <source>%1% available</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../greeter/enginedevice.cpp" line="301"/>
        <source>Left %1h %2m to full</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../greeter/enginedevice.cpp" line="303"/>
        <source>charging (%1%)</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../greeter/enginedevice.cpp" line="309"/>
        <source>%1 waiting to discharge (%2%)</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../greeter/enginedevice.cpp" line="314"/>
        <source>%1 waiting to charge (%2%)</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../greeter/enginedevice.cpp" line="334"/>
        <source>AC adapter</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../greeter/enginedevice.cpp" line="338"/>
        <source>Laptop battery</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../greeter/enginedevice.cpp" line="342"/>
        <source>UPS</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../greeter/enginedevice.cpp" line="346"/>
        <source>Monitor</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../greeter/enginedevice.cpp" line="350"/>
        <source>Mouse</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../greeter/enginedevice.cpp" line="354"/>
        <source>Keyboard</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../greeter/enginedevice.cpp" line="358"/>
        <source>PDA</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../greeter/enginedevice.cpp" line="362"/>
        <source>Cell phone</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../greeter/enginedevice.cpp" line="366"/>
        <source>Media player</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../greeter/enginedevice.cpp" line="370"/>
        <source>Tablet</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../greeter/enginedevice.cpp" line="374"/>
        <source>Computer</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../greeter/enginedevice.cpp" line="378"/>
        <source>unrecognised</source>
        <translation type="unfinished"></translation>
    </message>
</context>
<context>
    <name>FakeDialog</name>
    <message>
        <location filename="../common/fakedialog.cpp" line="63"/>
        <source>cancel</source>
        <translation type="unfinished"></translation>
    </message>
</context>
<context>
    <name>GreeterWindow</name>
    <message>
        <location filename="../greeter/greeterwindow.cpp" line="167"/>
        <source>LAN</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../greeter/greeterwindow.cpp" line="169"/>
        <source>WLAN</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../greeter/greeterwindow.cpp" line="385"/>
        <source>VirtualKeyboard</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../greeter/greeterwindow.cpp" line="387"/>
        <source>Power</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../greeter/greeterwindow.cpp" line="389"/>
        <source>SwitchUser</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../greeter/greeterwindow.cpp" line="391"/>
        <source>PowerInfo</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../greeter/greeterwindow.cpp" line="970"/>
        <source>Shut Down</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../greeter/greeterwindow.cpp" line="973"/>
        <source>Restart</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../greeter/greeterwindow.cpp" line="976"/>
        <source>Sleep</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../greeter/greeterwindow.cpp" line="979"/>
        <source>Hibernate</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../greeter/greeterwindow.cpp" line="1578"/>
        <location filename="../greeter/greeterwindow.cpp" line="2171"/>
        <source>Login</source>
        <translation type="unfinished">вход</translation>
    </message>
</context>
<context>
    <name>GreeterWrapper</name>
    <message>
        <location filename="../greeter/greeterwrapper.cpp" line="171"/>
        <source>failed to start session.</source>
        <translation>не начала сессии.</translation>
    </message>
</context>
<context>
    <name>IconEdit</name>
    <message>
        <location filename="../greeter/iconedit.cpp" line="137"/>
        <source>OK</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../greeter/iconedit.cpp" line="270"/>
        <location filename="../greeter/iconedit.cpp" line="318"/>
        <source>Password: </source>
        <translation type="unfinished">пароль: </translation>
    </message>
    <message>
        <location filename="../greeter/iconedit.cpp" line="272"/>
        <location filename="../greeter/iconedit.cpp" line="320"/>
        <source>Username</source>
        <translation type="unfinished"></translation>
    </message>
</context>
<context>
    <name>LettersWidget</name>
    <message>
        <location filename="../VirtualKeyboard/src/letterswidget.cpp" line="140"/>
        <source>&amp;&amp;?!</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../VirtualKeyboard/src/letterswidget.cpp" line="154"/>
        <source>123</source>
        <translation type="unfinished"></translation>
    </message>
</context>
<context>
    <name>LoginOptionsWidget</name>
    <message>
        <location filename="../greeter/loginoptionswidget.cpp" line="76"/>
        <location filename="../greeter/loginoptionswidget.cpp" line="99"/>
        <source>Login Options</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../greeter/loginoptionswidget.cpp" line="330"/>
        <source>Password</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../greeter/loginoptionswidget.cpp" line="356"/>
        <source>Other</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../greeter/loginoptionswidget.cpp" line="766"/>
        <source>Identify device removed!</source>
        <translation type="unfinished"></translation>
    </message>
</context>
<context>
    <name>LoginWindow</name>
    <message>
        <location filename="../greeter/loginwindow.cpp" line="320"/>
        <location filename="../greeter/loginwindow.cpp" line="783"/>
        <location filename="../greeter/loginwindow.cpp" line="1007"/>
        <location filename="../greeter/loginwindow.cpp" line="1008"/>
        <location filename="../greeter/loginwindow.cpp" line="1435"/>
        <source>Login</source>
        <translation>вход</translation>
    </message>
    <message>
        <source>logged in</source>
        <translation type="vanished">войти в систему</translation>
    </message>
    <message>
        <location filename="../greeter/loginwindow.cpp" line="1030"/>
        <source>login</source>
        <translation>вход</translation>
    </message>
    <message>
        <location filename="../greeter/loginwindow.cpp" line="1243"/>
        <source>Password: </source>
        <translation>пароль: </translation>
    </message>
    <message>
        <location filename="../greeter/loginwindow.cpp" line="1252"/>
        <source>Username</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../greeter/loginwindow.cpp" line="1389"/>
        <location filename="../greeter/loginwindow.cpp" line="1394"/>
        <source>Authentication failure, Please try again</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../greeter/loginwindow.cpp" line="538"/>
        <source>Verify face recognition or enter password to unlock</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../greeter/loginwindow.cpp" line="202"/>
        <location filename="../greeter/loginwindow.cpp" line="324"/>
        <source>Enter the ukey password</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../greeter/loginwindow.cpp" line="221"/>
        <location filename="../greeter/loginwindow.cpp" line="328"/>
        <source>Insert the ukey into the USB port</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../greeter/loginwindow.cpp" line="543"/>
        <source>Press fingerprint or enter password to unlock</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../greeter/loginwindow.cpp" line="548"/>
        <source>Verify voiceprint or enter password to unlock</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../greeter/loginwindow.cpp" line="553"/>
        <source>Verify finger vein or enter password to unlock</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../greeter/loginwindow.cpp" line="558"/>
        <source>Verify iris or enter password to unlock</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../greeter/loginwindow.cpp" line="563"/>
        <source>Use the bound wechat scanning code or enter the password to unlock</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../greeter/loginwindow.cpp" line="1247"/>
        <source>Input Password</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../greeter/loginwindow.cpp" line="1312"/>
        <source>Password cannot be empty</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../greeter/loginwindow.cpp" line="1386"/>
        <source>User name input error!</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../greeter/loginwindow.cpp" line="1444"/>
        <source>Retry</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../greeter/loginwindow.cpp" line="1732"/>
        <location filename="../greeter/loginwindow.cpp" line="1894"/>
        <location filename="../greeter/loginwindow.cpp" line="2036"/>
        <location filename="../greeter/loginwindow.cpp" line="2397"/>
        <source>Failed to verify %1, please enter password to unlock</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../greeter/loginwindow.cpp" line="1734"/>
        <location filename="../greeter/loginwindow.cpp" line="1896"/>
        <location filename="../greeter/loginwindow.cpp" line="2040"/>
        <location filename="../greeter/loginwindow.cpp" line="2042"/>
        <location filename="../greeter/loginwindow.cpp" line="2399"/>
        <source>Unable to verify %1, please enter password to unlock</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../greeter/loginwindow.cpp" line="2071"/>
        <source>Abnormal network</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../greeter/loginwindow.cpp" line="2056"/>
        <location filename="../greeter/loginwindow.cpp" line="2060"/>
        <source>Failed to verify %1, you still have %2 verification opportunities</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../greeter/loginwindow.cpp" line="2155"/>
        <location filename="../greeter/loginwindow.cpp" line="2156"/>
        <location filename="../greeter/loginwindow.cpp" line="2233"/>
        <location filename="../greeter/loginwindow.cpp" line="2234"/>
        <source>Please try again in %1 minutes.</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../greeter/loginwindow.cpp" line="2165"/>
        <location filename="../greeter/loginwindow.cpp" line="2166"/>
        <location filename="../greeter/loginwindow.cpp" line="2244"/>
        <location filename="../greeter/loginwindow.cpp" line="2245"/>
        <source>Please try again in %1 seconds.</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../greeter/loginwindow.cpp" line="2175"/>
        <location filename="../greeter/loginwindow.cpp" line="2176"/>
        <location filename="../greeter/loginwindow.cpp" line="2254"/>
        <location filename="../greeter/loginwindow.cpp" line="2255"/>
        <source>Account locked permanently.</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <source>Incorrect password, please input again</source>
        <translation type="obsolete">неверный пароль, пожалуйста, введите снова</translation>
    </message>
</context>
<context>
    <name>MainWindow</name>
    <message>
        <location filename="../display-switch/ui_mainwindow.h" line="127"/>
        <source>MainWindow</source>
        <translation type="unfinished"></translation>
    </message>
</context>
<context>
    <name>NumbersWidget</name>
    <message>
        <location filename="../VirtualKeyboard/src/numberswidget.cpp" line="143"/>
        <source>&amp;&amp;?!</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../VirtualKeyboard/src/numberswidget.cpp" line="157"/>
        <source>Return</source>
        <translation type="unfinished"></translation>
    </message>
</context>
<context>
    <name>PowerManager</name>
    <message>
        <location filename="../greeter/powerwindow.cpp" line="130"/>
        <source>Close all apps, turn off your computer, and then turn your computer back on</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../greeter/powerwindow.cpp" line="132"/>
        <source>Close all apps, and then shut down your computer</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../greeter/powerwindow.cpp" line="134"/>
        <source>The computer stays on, but consumes less power. The app stays open and can quickly wake up and revert to where you left off</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../greeter/powerwindow.cpp" line="137"/>
        <source>Turn off your computer, but the app stays open. When the computer is turned on, it can be restored to the state you left</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../greeter/powerwindow.cpp" line="396"/>
        <source>Switch User</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../greeter/powerwindow.cpp" line="397"/>
        <location filename="../greeter/powerwindow.cpp" line="453"/>
        <source>Restart</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../greeter/powerwindow.cpp" line="398"/>
        <source>Power Off</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../greeter/powerwindow.cpp" line="473"/>
        <source>Shut Down</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../greeter/powerwindow.cpp" line="494"/>
        <source>Hibernate</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../greeter/powerwindow.cpp" line="517"/>
        <source>Suspend</source>
        <translation type="unfinished"></translation>
    </message>
</context>
<context>
    <name>S:</name>
    <message>
        <location filename="../greeter/enginedevice.cpp" line="308"/>
        <source></source>
        <comment>tablet device</comment>
        <translation></translation>
    </message>
</context>
<context>
    <name>Surewidget</name>
    <message>
        <location filename="../greeter/surewidget.ui" line="14"/>
        <source>Form</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../greeter/surewidget.ui" line="56"/>
        <location filename="../greeter/surewidget.ui" line="66"/>
        <location filename="../greeter/surewidget.ui" line="79"/>
        <source>TextLabel</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../greeter/surewidget.ui" line="130"/>
        <source>Cancel</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../greeter/surewidget.ui" line="143"/>
        <source>Confirm</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../greeter/surewidget.cpp" line="21"/>
        <location filename="../greeter/surewidget.cpp" line="75"/>
        <source>Multiple users are logged in at the same time.Are you sure you want to reboot this system?</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../greeter/surewidget.cpp" line="40"/>
        <location filename="../greeter/surewidget.cpp" line="68"/>
        <source>There&apos;s a program running, this process cannot be interrupted.</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../greeter/surewidget.cpp" line="43"/>
        <source>The system will automatically %1 after the program is executed. Do not shut down the computer.</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../greeter/surewidget.cpp" line="72"/>
        <source>Click OK and the system will automatically %1 after the program is executed.</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../greeter/surewidget.cpp" line="111"/>
        <source>Shut Down</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../greeter/surewidget.cpp" line="114"/>
        <source>Restart</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../greeter/surewidget.cpp" line="117"/>
        <source>Sleep</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../greeter/surewidget.cpp" line="120"/>
        <source>Hibernate</source>
        <translation type="unfinished"></translation>
    </message>
</context>
<context>
    <name>TouchScreenSet</name>
    <message>
        <location filename="../common/touchscreenset.cpp" line="83"/>
        <source>%1</source>
        <translation type="unfinished"></translation>
    </message>
</context>
<context>
    <name>UsersModel</name>
    <message>
        <location filename="../greeter/usersmodel.cpp" line="48"/>
        <location filename="../greeter/usersmodel.cpp" line="50"/>
        <source>Guest</source>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../greeter/usersmodel.cpp" line="58"/>
        <source>Guest Session</source>
        <translation>гость.</translation>
    </message>
    <message>
        <location filename="../greeter/usersmodel.cpp" line="76"/>
        <location filename="../greeter/usersmodel.cpp" line="78"/>
        <location filename="../greeter/usersmodel.cpp" line="85"/>
        <location filename="../greeter/usersmodel.cpp" line="100"/>
        <source>Login</source>
        <translation>вход</translation>
    </message>
</context>
</TS>
